#!/bin/bash
#
# Rename an e-smith package as smeserver and change relevant files
# $1 = Module name e.g. e-smith-dnscache
# $2-$5 can appear in any order and are optional 
#    = Organisation (smeserver or user - defaults to smeserver)
#    = "local" will use parameters set for local repository else it will use remote
#    = "force" will automagically delete an existing repo, otherwise it will prompt
#    = "debug" run in debug mode 
# This works whether the repo and local files exist of not (it deletes them if necessary)
#

if [[ -z $1 ]] ; then
	echo "Rename an e-smith package as smeserver and change relevant files"
	echo "rename-e-smith-pkg.sh <modulename> [<organization> <local> <force> <debug>]"
  echo "<modulename> = Module/package name (e.g. e-smith-dnscache)"
  echo " can appear in any order and are optional"
  echo "   - <organisation> (smeserver, smecontribs or user - defaults to smeserver)"
  echo "   - <local> will use parameters set for local repository, else it will use remote"
  echo "   - <force> will automagically delete an existing repo, otherwise it will prompt"
  echo "   - <debug> run in debug mode"
  echo "   - <nooutput> completely suppress all output"
  exit 0
fi

#
# Pull in parameters from a config file ~/.smegit/config
#
inifilename=$(echo ~)"/.smegit/config"
if [ ! -e $inifilename ] ; then
  # Not here, look at system default
  if [ ! -e /etc/smegit.ini ] ; then 
    echo "No ini file found $inifiename or /etc/smegit.ini"
    echo "git-cvs2git.sh <modulename> <organization> [<local>]"
    exit 1
  else
    initfilename="/etc/smegit.ini"
  fi
fi

while read -r line || [[ -n "$line" ]]; do
    if [[ $line =~ ^\[.*\]$ ]]
    then
        section=${line#*[}
        section=${section%]*}
    else
        if [[ $line =~ ^[^#]*= ]]
        then
            key=${line%=*}
            value=${line#*=}
            declare "${section}_${key}=$value"
        fi
    fi
done < "$inifilename"
DEBUG=
if [ ${smegit_DEBUG} == "true" ] ; then DEBUG=true ; fi
if [ $DEBUG ] ; then echo "************found ini file: $inifilename" ; fi

if [[ $smegit_WORKDIR == ~* ]] ; then
    # relative to users home dir
    WORKDIR=$(echo ~)${smegit_WORKDIR:1}
else
    # absolute path
    WORKDIR=${smegit_WORKDIR}
fi

SOURCEHOST=${remote_GITEAHOST}
SOURCEACCESSTOKEN=${remote_GITEAACCESSTOKEN}
SOURCEORG="smeserver"
TARGETORG=${remote_USER}
DELETEIT=
NOOUTPUT=
for param in $2 $3 $4 $5 $6; do
  if [ $param ] ; then
    case $param in
      local )
	      SOURCEHOST=${local_GITEAHOST}
        SOURCEACCESSTOKEN=${local_GITEAACCESSTOKEN}
        TARGETORG=${local_USER} ;;
      force )
        DELETEIT=true ;;
      debug )
        DEBUG=true ;;
      nooutput )
        NOOUTPUT=true ;;
      * )
        SOURCEORGORG=$param ;;
    esac
  else
	  break
  fi
done
# Debug settings
# Make this null if you want lots of output. Still quite a bit anyway
QUIET="--quiet"
SILENT="-s"
if [ $DEBUG ] ; then 
	QUIET=
	SILENT="-v"
fi
# Make this null to get curl to check ssl cert
checkSSL="-k"

#Extract <pkg>
SOURCEPKG=$1
TARGETPKG=${SOURCEPKG/e-smith/smeserver}

#Check that source package exists
if [ $DEBUG ] ; then echo "Check if $SOURCEORG/$SOURCEPKG is there!" ; fi
RESPONSE=$(curl $SILENT -o /dev/null -w "%{http_code}" "$SOURCEHOST/api/v1/repos/$SOURCEORG/$SOURCEPKG")
if [ "$RESPONSE" == "200" ]; then
    if [ $DEBUG ] ; then echo "Repository for $SOURCEORG/$SOURCEPKG exists!" ; fi
else
    echo "************Repository for $SOURCEORG/$SOURCEPKG does not exist on $SOURCEHOST ($RESPONSE)"
    exit 1
fi

#Check that target package does not exist
if [ $DEBUG ] ; then echo "Check if $TARGETORG/$TARGETPKG is there!" ; fi
RESPONSE=$(curl $SILENT -o /dev/null -w "%{http_code}" "$SOURCEHOST/api/v1/repos/$TARGETORG/$TARGETPKG")
if [ "$RESPONSE" == "200" ]; then
  if [ $DEBUG ] ; then echo "Repository for $TARGETORG/$TARGETPKG exists!" ; fi
  if [ -z ${DELETEIT} ] ; then
    while true; do
		  read -p "Do you wish to delete it and continue?(y/n) " yn
		  case $yn in
			  [Yy]* ) 
                DELETEIT=true
                break ;;
			  [Nn]* )
                echo "************Abandoning Fork of $SOURCEORG/$SOURCEPKG on $SOURCEHOST ($RESPONSE)" 
                exit 1 ;;
			  * ) echo "Please answer yes or no.";;
		  esac
	  done
  fi
  if [ $DELETEIT ] ; then
    if [ $DEBUG ] ; then echo "Deleting $TARGETORG/$TARGETPKG" ; fi
    RESPONSE=$(curl "$checkSSL" "$SILENT" -X  'DELETE' \
      "$SOURCEHOST/api/v1/repos/$TARGETORG/$TARGETPKG" \
      -H 'accept: application/json' \
      -H "Authorization: token $SOURCEACCESSTOKEN" )
  fi
else
    if [ $DEBUG ] ; then echo "Repository for $TARGETORG/$TARGETPKG does not exist on $SOURCEHOST" ; fi
fi

#Fork e-smith-<pkg> into users repositories as smeserver-<pkg>
# - add bugzilla and wiki references

# create a Bugzilla URL
PRODUCTBUGZILLA="SME%20Server%2010.X"
BASEORCONTRIB="base"
if [[ "$SOURCEORG" == "smecontribs" ]]; then
  PRODUCTBUGZILLA="SME%20Contribs"
  BASEORCONTRIB="contrib"
fi  

# grab WIKILINK from source package
#WIKILINK=$(curl "$SILENT" -X 'GET' \
#  "$SOURCEHOST/api/v1/repos/$SOURCEORG/$SOURCEPKG" \
#  -H 'accept: application/json' \
#  -H 'Content-Type: application/json' \
#  -H "Authorization: token $SOURCEACCESSTOKEN" | \
#  jq -r ' .external_wiki.external_wiki_url')
#if [ $DEBUG ] ; then echo "WIKILINK=$WIKILINK" ; fi

# migrate the source package over to target with updated fields
if [ $DEBUG ] ; then echo "Migrating $SOURCEORG/$SOURCEPKG as $TARGETORG/$1 on $TARGETHOST" ; fi
RESPONSE=$(curl $SILENT -k -X 'POST' \
  "$SOURCEHOST/api/v1/repos/migrate" \
  -H 'accept: application/json' \
  -H "Authorization: token $SOURCEACCESSTOKEN" \
  -H 'Content-Type: application/json' \
  -d '{
    "auth_token": "'"$SOURCEACCESSTOKEN"'",
    "description": "'"SMEServer Koozali developed git repo for $TARGETPKG $BASEORCONTRIB"'",
    "clone_addr": "'"$SOURCEHOST/$TARGETORG/$TARGETPKG.git"'",
    "repo_name": "'"$TARGETPKG"'",
    "repo_owner": "'"$TARGETORG"'",
    "external_tracker": {
		  "external_tracker_url": "'"https://bugs.koozali.org/buglist.cgi?component=$TARGETPKG&product=$PRODUCTBUGZILLA"'"
		  }
    }'
  )

# Update the repo with the changed description, bugzilla url and wiki url
#if [ $DEBUG ] ; then echo "Updating Description, Bug and Wiki links" ; fi
#RESPONSE=$(curl "$SILENT" "$checkSSL" -X 'PATCH' \
#    "$SOURCEHOST/api/v1/repos/$TARGETORG/$TARGETPKG"   \
#	-H 'accept: application/json'   \
#	-H "Authorization: token $SOURCEACCESSTOKEN" \
#	-H 'Content-Type: application/json'   \
#	-d '{
#			"description": "'"SMEServer Koozali developed repo for $TARGETPKG base"'",
#      "has_issues": true,
#      "external_tracker": {
#		    "external_tracker_format": "https://bugs.koozali.org/show_bug.cgi?id={index}",
#			  "external_tracker_style": "numeric",
#			  "external_tracker_url": "'"https://bugs.koozali.org/buglist.cgi?component=$TARGETPKG&product=$PRODUCTBUGZILLA"'"
#			},
#      "has_wiki": true,
#			"external_wiki": {
#				"external_wiki_url": "'"$WIKILINK"'"
#			}
#		}'
#	)

# Create the local Git repository
GITFiles=$WORKDIR/GITFiles/$TARGETORG
mkdir -p $GITFiles
cd $GITFiles

# Delete the local first
if [ -d "$TARGETPKG" ] ; then
	if [ $DEBUG ] ; then echo "Deleting local GIT files" ; fi
	rm -Rf $GITFiles/$TARGETPKG
fi

# Clone the package
if [ $DEBUG ] ; then echo "git clone $SOURCEHOST/$TARGETORG/$TARGETPKG.git" ; fi
cd $GITFiles
git clone "$SOURCEHOST/$TARGETORG/$TARGETPKG.git" $QUIET 
cd $GITFiles/$TARGETPKG

#Update README.md
# - new name
sed -i 's/e-smith/smeserver/g' README.md

# - Bugzilla link to new name whilst keeping old link for Legacy
sed -i '/Show list/p' README.md
sed -i "/Show list/{ n; s/Show list of outstanding bugs\:/And a list of outstanding Legacy bugs\: ($SOURCEPKG)/}" README.md
sed -i '/Show list/{ n; s/smeserver-/e-smith-/g}' README.md
sed -i '/Show list/ s/)/)\\/' README.md

git add README.md

#Update Makefile
# - NAME := smeserver-<pkg>

sed -i 's/e-smith/smeserver/g' Makefile
git add Makefile

# spec file
#Rename e-smith-<pkg>.spec as smeserver-<pkg>.spec
git mv "$SOURCEPKG.spec" "$TARGETPKG.spec"

#Update smeserver-<pkg>.spec
# - change name to smeserver-<pkg>
sed -i "/^Summary:/ s/e-smith/smeserver/" "$TARGETPKG.spec"
sed -i "s/^%define name.*$SOURCEPKG/%define name $TARGETPKG/" "$TARGETPKG.spec"
sed -i "/^%description/{ n; s/e-smith/smeserver/}" "$TARGETPKG.spec"

# - change all Requires: and BuildRequires: e-smith* to smeserver*
sed -i 's/^Requires:.*e-smith/Requires: smeserver/g' "$TARGETPKG.spec"
sed -i 's/^BuildRequires:.*e-smith/BuildRequires: smeserver/g' "$TARGETPKG.spec"

# - add changelog entry (Bugzilla #12359)
# Bump the release and add changelog
change-log "$TARGETPKG.spec" >/dev/null 2>&1
# and edit in the reason and add by whom
sed -i "s/fix \[SME: \]/Rename to $TARGETPKG \[SME: 12359\]/" "$TARGETPKG.spec"
sed -i 's/ME MYSELF <myemail@koozali.org>/rename-e-smith-pkg.sh by Trevor Batley <trevor@batley.id.au>/' "$TARGETPKG.spec"

# - add PROVIDES: smeserver-<pkg>
sed -i "/%description/i Provides: $SOURCEPKG" "$TARGETPKG.spec"

# - change Source: to new standard of tar.gz
sed -i "/^Source:/ s/tar.xz/tar.gz/" "$TARGETPKG.spec"

# tell git about the changes
git add "$TARGETPKG.spec"

#Delete archivefilename file if it follows the new standard
if [ -e archivefilename ] ; then git rm archivefilename $QUIET ; fi

#Update createlinks
# - if there is an e-smith-<pkg>-update event rename to smeserver-<pkg>-update and add symlink

git commit -m "rename-e-smith-pkg script (#12359)" $QUIET
git push origin &> /dev/null

#Convert repository to a Regular Repository (rather than fork)
#Can't do via API call ATM, so leave a message
if [[ -z $NOOUTPUT ]] ; then 
  echo "$SOURCEORG/$SOURCEPKG has been renamed as $TARGETORG/$TARGETPKG"
  echo "It is still a forked repository, so you need to 'Convert to Regular Repository' in GITEA and then 'Transfer' it to it's final Organisation"
fi
exit 0
